#include "pwm-timer.hpp"

PwmTimer::PwmTimer(TIM_TypeDef* timer,
                   OutputCompareChannel OCC,
                   uint16_t pwmResolution,
                   uint16_t pwmFrequency,
                   uint16_t initalDutyCycle)
    : _timer(timer), _OCC(OCC), _pwmResolution(pwmResolution), _pwmFrequency(pwmFrequency) {
	enableTimer();

	CLEAR_BIT(timer->CR1, TIM_CR1_DIR);   // upcounting mode
	CLEAR_BIT(timer->CR1, TIM_CR1_ARPE);  // enable auto-reload

	setDutyCycle(initalDutyCycle);  // Reset PWM
	setupOutputChannel();           // Configure Output Channel

	WRITE_REG(timer->ARR, pwmResolution);  // PWM resolution

	// TODO - Variable for clock speed
	uint16_t prescaler = 8000000 / (pwmResolution * pwmFrequency);
	WRITE_REG(timer->PSC, prescaler);  // Prescaler for timer counter
	SET_BIT(_timer->BDTR, TIM_BDTR_MOE);
	SET_BIT(timer->CR1, TIM_CR1_CEN);  // Enable counter
}

PwmTimer::PwmTimer(TIM_TypeDef* timer, OutputCompareChannel OCC, uint16_t pwmResolution, uint16_t pwmFrequency)
    : PwmTimer(timer, OCC, pwmResolution, pwmFrequency, 0) {}

void PwmTimer::setDutyCycle(uint16_t compareValue) {
	compareValue = compareValue < _pwmResolution ? compareValue : _pwmResolution;
	switch (_OCC) {
		case OutputCompareChannel::OCC1:
			_timer->CCR1 = compareValue;
			break;
		case OutputCompareChannel::OCC2:
			_timer->CCR2 = compareValue;
			break;
		case OutputCompareChannel::OCC3:
			_timer->CCR3 = compareValue;
			break;
		case OutputCompareChannel::OCC4:
			_timer->CCR4 = compareValue;
			break;
	}
}

void PwmTimer::setDutyCyclePercentage(uint8_t dutyCyclePercentage) {
	dutyCyclePercentage = dutyCyclePercentage <= 100 ? dutyCyclePercentage : 100;
	uint16_t captureCompareValue = _pwmResolution * dutyCyclePercentage / 100;
	this->setDutyCycle(captureCompareValue);
}

void PwmTimer::enableTimer() {
	if (_timer == TIM1) {
		SET_BIT(RCC->APB2ENR, RCC_APB2ENR_TIM1EN);
	} else if (_timer == TIM2) {
		SET_BIT(RCC->APB1ENR, RCC_APB1ENR_TIM2EN);
	} else if (_timer == TIM3) {
		SET_BIT(RCC->APB1ENR, RCC_APB1ENR_TIM3EN);
	} else if (_timer == TIM14) {
		SET_BIT(RCC->APB1ENR, RCC_APB1ENR_TIM14EN);
	} else if (_timer == TIM15) {
		SET_BIT(RCC->APB2ENR, RCC_APB2ENR_TIM15EN);
	} else if (_timer == TIM16) {
		SET_BIT(RCC->APB2ENR, RCC_APB2ENR_TIM16EN);
	} else if (_timer == TIM17) {
		SET_BIT(RCC->APB2ENR, RCC_APB2ENR_TIM17EN);
	}
}

void PwmTimer::setupOutputChannel() {
	switch (_OCC) {
		case OutputCompareChannel::OCC1:
			CLEAR_BIT(_timer->CCER, TIM_CCER_CC1E);    // CCxS bits are writable only when the channel is off
			CLEAR_BIT(_timer->CCMR1, TIM_CCMR1_CC1S);  // set chanel as output
			SET_BIT(_timer->CCER, TIM_CCER_CC1E);      // Channel is active high
			SET_BIT(_timer->CCMR1, TIM_CCMR1_OC1M_2 | TIM_CCMR1_OC1M_1);  // enable pwm mode 1
			CLEAR_BIT(_timer->CCER, TIM_CCER_CC1P);                       // enable output signal polarity of channel
			break;
		case OutputCompareChannel::OCC2:
			CLEAR_BIT(_timer->CCER, TIM_CCER_CC2E);
			CLEAR_BIT(_timer->CCMR1, TIM_CCMR1_CC2S);
			SET_BIT(_timer->CCER, TIM_CCER_CC2E);
			SET_BIT(_timer->CCMR1, TIM_CCMR1_OC2M_2 | TIM_CCMR1_OC2M_1);
			CLEAR_BIT(_timer->CCER, TIM_CCER_CC2P);
			break;
		case OutputCompareChannel::OCC3:
			CLEAR_BIT(_timer->CCER, TIM_CCER_CC3E);
			CLEAR_BIT(_timer->CCMR2, TIM_CCMR2_CC3S);
			SET_BIT(_timer->CCER, TIM_CCER_CC3E);
			SET_BIT(_timer->CCMR2, TIM_CCMR2_OC3M_2 | TIM_CCMR2_OC3M_1);
			CLEAR_BIT(_timer->CCER, TIM_CCER_CC3P);
			break;
		case OutputCompareChannel::OCC4:
			CLEAR_BIT(_timer->CCER, TIM_CCER_CC4E);
			CLEAR_BIT(_timer->CCMR2, TIM_CCMR2_CC4S);
			SET_BIT(_timer->CCER, TIM_CCER_CC4E);
			SET_BIT(_timer->CCMR2, TIM_CCMR2_OC4M_2 | TIM_CCMR2_OC4M_1);
			CLEAR_BIT(_timer->CCER, TIM_CCER_CC4P);
			break;
	}
}
